"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const events_1 = require("events");
class ApiEmitter extends events_1.EventEmitter {
    constructor(options, session) {
        super();
        this.options = options;
        this.session = session;
    }
    handleMessage(event) {
        if (event.deltaNewMessage) {
            const delta = event.deltaNewMessage;
            const message = {
                threadId: this.getThreadId(delta),
                attachments: delta.attachments,
                authorId: delta.messageMetadata.actorFbId,
                id: delta.messageMetadata.messageId,
                timestamp: delta.messageMetadata.timestamp,
                message: delta.body || "",
                stickerId: delta.stickerId
            };
            if (message.authorId === this.session.tokens.uid && !this.options.selfListen)
                return;
            this.emit("message", message);
            return;
        }
        if (event.deltaThreadName) {
            const delta = event.deltaThreadName;
            const threadNameEvent = Object.assign({}, this.getEventMetadata(delta), { name: delta.name });
            this.emit("threadNameEvent", threadNameEvent);
            this.emit("event", { event: threadNameEvent, type: "threadNameEvent" });
            return;
        }
        if (event.deltaDeliveryReceipt) {
            const delta = event.deltaDeliveryReceipt;
            const deliveryReceiptEvent = {
                threadId: this.getThreadId(delta),
                receiverId: delta.actorFbId || this.getThreadId(delta)
            };
            this.emit("deliveryReceiptEvent", deliveryReceiptEvent);
            this.emit("event", { event: deliveryReceiptEvent, type: "deliveryReceiptEvent" });
            return;
        }
        if (event.deltaReadReceipt) {
            const delta = event.deltaReadReceipt;
            const readReceiptEvent = {
                threadId: this.getThreadId(delta),
                receiverId: delta.actorFbId || this.getThreadId(delta)
            };
            this.emit("readReceiptEvent", readReceiptEvent);
            this.emit("event", { event: readReceiptEvent, type: "readReceiptEvent" });
            return;
        }
        if (event.deltaParticipantsAddedToGroupThread) {
            const delta = event.deltaParticipantsAddedToGroupThread;
            const participantsAddedToGroupThreadEvent = Object.assign({}, this.getEventMetadata(delta), { participantIds: delta.addedParticipants.map(user => user.userFbId) });
            this.emit('participantsAddedToGroupThreadEvent', participantsAddedToGroupThreadEvent);
            this.emit("event", { event: participantsAddedToGroupThreadEvent, type: "participantsAddedToGroupThreadEvent" });
            return;
        }
        if (event.deltaParticipantLeftGroupThread) {
            const delta = event.deltaParticipantLeftGroupThread;
            const participantLeftGroupThreadEvent = Object.assign({}, this.getEventMetadata(delta), { participantId: delta.leftParticipantFbId });
            this.emit('participantLeftGroupThreadEvent', participantLeftGroupThreadEvent);
            this.emit("event", { event: participantLeftGroupThreadEvent, type: "participantLeftGroupThreadEvent" });
            return;
        }
        if (event.deltaAdminTextMessage) {
            const delta = event.deltaAdminTextMessage;
            switch (delta.type) {
                case 'change_thread_nickname':
                    const changeThreadNicknameEvent = Object.assign({}, this.getEventMetadata(delta), { participantId: delta.untypedData.participant_id, nickname: delta.untypedData.nickname });
                    this.emit('changeThreadNicknameEvent', changeThreadNicknameEvent);
                    this.emit("event", { event: changeThreadNicknameEvent, type: "changeThreadNicknameEvent" });
                    break;
                case 'change_thread_admins':
                    const addThreadAdminsEvent = Object.assign({}, this.getEventMetadata(delta), { participantId: delta.untypedData.TARGET_ID });
                    this.emit('addThreadAdminsEvent', addThreadAdminsEvent);
                    this.emit("event", { event: addThreadAdminsEvent, type: "addThreadAdminsEvent" });
                    break;
                case 'lightweight_event_create':
                    const eventCreateEvent = Object.assign({}, this.getEventMetadata(delta), this.getFacebookEventMetadata(delta.untypedData));
                    this.emit('eventCreateEvent', eventCreateEvent);
                    this.emit("event", { event: eventCreateEvent, type: "eventCreateEvent" });
                    break;
                case 'lightweight_event_update_title':
                    const eventUpdateTitleEvent = Object.assign({}, this.getEventMetadata(delta), this.getFacebookEventMetadata(delta.untypedData));
                    this.emit('eventUpdateTitleEvent', eventUpdateTitleEvent);
                    this.emit("event", { event: eventUpdateTitleEvent, type: "eventUpdateTitleEvent" });
                    break;
                case 'lightweight_event_update_time':
                    const eventUpdateTimeEvent = Object.assign({}, this.getEventMetadata(delta), this.getFacebookEventMetadata(delta.untypedData));
                    this.emit('eventUpdateTimeEvent', eventUpdateTimeEvent);
                    this.emit("event", { event: eventUpdateTimeEvent, type: "eventUpdateTimeEvent" });
                    break;
                case 'lightweight_event_update_location':
                    const eventUpdateLocationEvent = Object.assign({}, this.getEventMetadata(delta), this.getFacebookEventMetadata(delta.untypedData));
                    this.emit('eventUpdateLocationEvent', eventUpdateLocationEvent);
                    this.emit("event", { event: eventUpdateLocationEvent, type: "eventUpdateLocationEvent" });
                    break;
                case 'lightweight_event_rsvp':
                    const eventRsvpEvent = Object.assign({}, this.getEventMetadata(delta), this.getFacebookEventMetadata(delta.untypedData), { guestId: Number(delta.untypedData.guest_id), status: delta.untypedData.guest_status });
                    this.emit('eventRsvpEvent', eventRsvpEvent);
                    this.emit("event", { event: eventRsvpEvent, type: "eventRsvpEvent" });
                    break;
                case 'lightweight_event_delete':
                    const eventDeleteEvent = Object.assign({}, this.getEventMetadata(delta), this.getFacebookEventMetadata(delta.untypedData));
                    this.emit('eventDeleteEvent', eventDeleteEvent);
                    this.emit("event", { event: eventDeleteEvent, type: "eventDeleteEvent" });
                    break;
                case 'group_poll':
                    switch (delta.untypedData.event_type) {
                        case 'question_creation':
                            const pollCreateEvent = Object.assign({}, this.getEventMetadata(delta), this.getPollMetadata(delta.untypedData));
                            this.emit('pollCreateEvent', pollCreateEvent);
                            this.emit("event", { event: pollCreateEvent, type: "pollCreateEvent" });
                            break;
                        case 'update_vote':
                            const pollUpdateVoteEvent = Object.assign({}, this.getEventMetadata(delta), this.getPollMetadata(delta.untypedData));
                            this.emit('pollUpdateVoteEvent', pollUpdateVoteEvent);
                            this.emit("event", { event: pollUpdateVoteEvent, type: "pollUpdateVoteEvent" });
                            break;
                    }
                    break;
            }
        }
    }
    getEventMetadata(delta) {
        return {
            id: delta.messageMetadata.messageId,
            threadId: this.getThreadId(delta),
            authorId: delta.messageMetadata.actorFbId,
            message: delta.messageMetadata.adminText
        };
    }
    getThreadId(delta) {
        const { threadKey } = delta.messageMetadata || delta;
        return threadKey.threadFbId || threadKey.otherUserFbId;
    }
    getFacebookEventMetadata(untypedData) {
        return {
            creatorId: Number(untypedData.event_creator_id),
            title: untypedData.event_title,
            time: new Date(untypedData.event_time * 1000),
            location: untypedData.event_location_name,
            guests: JSON.parse(untypedData.guest_state_list).map(guest => ({
                state: guest.guest_list_state,
                id: guest.node.id
            }))
        };
    }
    getPollMetadata(untypedData) {
        let question = JSON.parse(untypedData.question_json);
        return {
            title: question.text,
            options: question.options.map(option => ({
                id: Number(option.id),
                title: option.text,
                voteCount: option.total_count,
                voters: option.voters.map(Number),
                viewerHasVoted: option.viewer_has_voted === 'true'
            }))
        };
    }
}
exports.default = ApiEmitter;
//# sourceMappingURL=ApiEmitter.js.map